/*

Plutt que de dcoder directement la quadrature,
on utilise des bascules pour dtecter le sens du
mouvement, et un diviseur de frquence pour avoir
le temps de compter.

//Requis :
#define ENC_D_COUNT_DDR
#define ENC_D_COUNT_PORTIN
#define ENC_D_COUNT_PIN
#define ENC_D_SENS_DDR
#define ENC_D_SENS_PORTIN
#define ENC_D_SENS_PIN
#define ENC_G_COUNT_DDR
#define ENC_G_COUNT_PORTIN
#define ENC_G_COUNT_PIN
#define ENC_G_SENS_DDR
#define ENC_G_SENS_PORTIN
#define ENC_G_SENS_PIN
*/

#if !defined(__LIB_ENCODEURS__)

	#define __LIB_ENCODEURS__

//******************************
// Librairies
//******************************

#include "bitmanip.c"

//******************************
// Defines
//******************************

#ifndef GAUCHE
	#define GAUCHE 0
#endif
#ifndef DROITE
	#define DROITE 1
#endif

//******************************
// Types de variables
//******************************

typedef struct struct_EncoderState {
	signed   long position;
	unsigned char retourzero;
} EncoderStateType;

//******************************
// Variables globales
//******************************

volatile EncoderStateType EncoderState[2];

//******************************
// Fonctions
//******************************

void Encodeurs_Init() {

	EncoderState[GAUCHE].position = 0;
	EncoderState[DROITE].position = 0;

	// Configuration en entre des ports
	cbi(ENC_G_COUNT_DDR, ENC_G_COUNT_PIN);
	cbi(ENC_G_SENS_DDR,  ENC_G_SENS_PIN);
	cbi(ENC_D_COUNT_DDR, ENC_D_COUNT_PIN);
	cbi(ENC_D_SENS_DDR,  ENC_D_SENS_PIN);
}
signed long Encodeurs_GetPosition(unsigned char encoderNum) {

	if(encoderNum < 2)
		return EncoderState[encoderNum].position;
	else
		return 0;
}
void Encodeurs_SetPosition(unsigned char encoderNum, signed long position) {

	if(encoderNum < 2)
		EncoderState[encoderNum].position = position;
}
void Encodeurs_Interrupt() {

	/* Il faut faire attention  ne pas sauter d'impulsions
	 * si l'on appelle trop lentement cette fonction
	 *
	 * Ici, on ne comptera qu'une seule des phases divise en frquence
	 * La deuxime sert uniquement pour le sens avec des bascules
	 */

	if (EncoderState[GAUCHE].retourzero && bit_is_set(ENC_G_COUNT_PORTIN, ENC_G_COUNT_PIN) && bit_is_clear(ENC_G_SENS_PORTIN, ENC_G_SENS_PIN)) {
		EncoderState[GAUCHE].position++;
		EncoderState[GAUCHE].retourzero = 0;
	} else if (EncoderState[GAUCHE].retourzero && bit_is_set(ENC_G_COUNT_PORTIN, ENC_G_COUNT_PIN) && bit_is_set(ENC_G_SENS_PORTIN, ENC_G_SENS_PIN)) {
		EncoderState[GAUCHE].position--;
		EncoderState[GAUCHE].retourzero = 0;
	} else if (bit_is_clear(ENC_G_COUNT_PORTIN, ENC_G_COUNT_PIN)) {
		EncoderState[GAUCHE].retourzero = 1;
	}

/*	if (EncoderState[DROITE].retourzero && bit_is_set(ENC_D_COUNT_PORTIN, ENC_D_COUNT_PIN) && bit_is_set(ENC_D_SENS_PORTIN, ENC_D_SENS_PIN)) {
		EncoderState[DROITE].position++;
		EncoderState[DROITE].retourzero = 0;
	} else if (EncoderState[DROITE].retourzero && bit_is_set(ENC_D_COUNT_PORTIN, ENC_D_COUNT_PIN) && bit_is_clear(ENC_D_SENS_PORTIN, ENC_D_SENS_PIN)) {
		EncoderState[DROITE].position--;
		EncoderState[DROITE].retourzero = 0;
	} else if (bit_is_clear(ENC_D_COUNT_PORTIN, ENC_D_COUNT_PIN)) {
		EncoderState[DROITE].retourzero = 1;
	}*/

}

#endif
