/*
 *
 * #define BUFFER_SIZE
 *
 * Buffer_Init(unsigned char *start, unsigned short size);
 *
 * while (Buffer_IsNotEmpty()) {
 * 	element = Buffer_GetFromFront();
 * }
 *
 * element = Buffer_GetAtIndex(unsigned short index);
 *
 * data = 0xAB;
 * if (Buffer_IsNotFull())
 * 	Buffer_AddToEnd(unsigned char data);
 *
 * Buffer_DumpFromFront(unsigned short numbytes);
 *
 * Buffer_Flush();
 *
 */

#if !defined(__LIB_BUFFERSERIE__)

	#define __LIB_BUFFERSERIE__

//******************************
// Types de variables
//******************************

struct buffer {
	unsigned char *dataptr;      // Pointeur vers la zone mmoire de stockage du buffer
	unsigned int   size;         // La taille alloue au buffer
	unsigned int   datalength;   // La longueur actuellement utilise dans le buffer
	unsigned int   dataindex;    // L'index actuel
} bufferserie;

//******************************
// Variables globales
//******************************

unsigned char Buffer_tab[BUFFER_SIZE];

//******************************
// Fonctions
//******************************

void Buffer_Init() {
	bufferserie.dataptr = Buffer_tab;
	bufferserie.size = BUFFER_SIZE;
	bufferserie.dataindex = 0;
	bufferserie.datalength = 0;
}
unsigned char Buffer_GetFromFront() {

	unsigned char data = 0;

	// Y a-t-il des donnes dans le buffer ?
	if (bufferserie.datalength) {
		// Prend le premier lment du buffer
		data = bufferserie.dataptr[bufferserie.dataindex];
		// Ractualise l'index et la taille du buffer
		bufferserie.dataindex++;
		if (bufferserie.dataindex >= bufferserie.size) {
			bufferserie.dataindex %= bufferserie.size;
		}
		bufferserie.datalength--;
	}
	return data;
}
void Buffer_DumpFromFront(unsigned short numbytes) {
	// Efface les numbytes premiers lments du buffer
	// Efface-t-on moins que le buffer entier ?
	if (numbytes < bufferserie.datalength) {
		// Ractualise l'index et la taille du buffer
		bufferserie.dataindex += numbytes;
		if (bufferserie.dataindex >= bufferserie.size) {
			bufferserie.dataindex %= bufferserie.size;
		}
		bufferserie.datalength -= numbytes;
	} else {
		// Efface le buffer tout entier
		bufferserie.datalength = 0;
	}
}
unsigned char Buffer_GetLength() {
	return bufferserie.datalength;
}
unsigned char Buffer_GetAtIndex(unsigned short index) {
	return bufferserie.dataptr[(bufferserie.dataindex+index) % bufferserie.size];
}
unsigned char Buffer_AddToEnd(unsigned char data) {
	// On s'assure qu'il reste de la place
	if (bufferserie.datalength < bufferserie.size) {
		// Ajout des donnes en fin de buffer
		bufferserie.dataptr[(bufferserie.dataindex + bufferserie.datalength) % bufferserie.size] = data;
		// Augmente la taille du buffer
		bufferserie.datalength++;
		return 1;
	} else {
		return 0;
	}
}
unsigned char Buffer_IsNotFull() {
	// Contrle si le buffer a encore de la place
	return (bufferserie.datalength < bufferserie.size);
}
unsigned char Buffer_IsNotEmpty() {
	// Contrle si le buffer a encore des lments
	return (bufferserie.datalength > 0);
}
void Buffer_Flush() {
	// Efface tout le contenu du buffer
	bufferserie.datalength = 0;
}

#endif
